<?php

namespace Drupal\poll_private\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\poll\Entity\Poll;
use Drupal\Component\Utility\Crypt;

class PollPrivateManageForm extends FormBase {

  public function getFormId() {
    return 'poll_private_manage_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state, Poll $poll = null) {
    // Almacenar el ID de la encuesta para usar en submit
    $form_state->set('poll_id', $poll->id());
    // Verificar si ya es privada
    $is_private = \Drupal::database()->select('poll_private_tokens', 'ppt')
      ->condition('poll_id', $poll->id())
      ->countQuery()
      ->execute()
      ->fetchField();

    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Make this poll private'),
      '#default_value' => (bool) $is_private,
    ];

    $form['emails'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Allowed emails'),
      '#description' => $this->t('Enter one email per line. Each will receive a unique access token.'),
      '#states' => [
        'visible' => [
          ':input[name="status"]' => ['checked' => TRUE],
        ],
      ],
    ];

    if ($is_private) {
      $form['current_tokens'] = $this->buildTokenTable($poll->id());
    }

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Save settings'),
      ],
    ];

    return $form;
  }

  protected function buildTokenTable($poll_id) {
    $tokens = \Drupal::database()->select('poll_private_tokens', 'ppt')
      ->fields('ppt', ['id', 'email', 'token', 'used'])
      ->condition('poll_id', $poll_id)
      ->execute()
      ->fetchAll();

    $header = [
      $this->t('Email'),
      $this->t('Token'),
      $this->t('Status'),
      $this->t('Operations'),
    ];

    $rows = [];
    foreach ($tokens as $token) {
      $rows[] = [
        $token->email,
        $token->token,
        $token->used ? $this->t('Used') : $this->t('Active'),
        [
          'data' => [
            '#type' => 'operations',
            '#links' => [
              'resend' => [
                'title' => $this->t('Resend'),
                'url' => \Drupal\Core\Url::fromRoute('poll_private.token_resend', [
                  'token_id' => $token->id,
                ]),
              ],
              'delete' => [
                'title' => $this->t('Delete'),
                'url' => \Drupal\Core\Url::fromRoute('poll_private.token_delete', [
                  'token_id' => $token->id,
                ]),
              ],
            ],
          ],
        ],
      ];
    }

    return [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No tokens generated yet.'),
      '#prefix' => '<h3>' . $this->t('Current access tokens') . '</h3>',
    ];
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    $poll_id = $form_state->get('poll_id');
    $database = \Drupal::database();

    if (!$form_state->getValue('status')) {
      // Si se desmarca como privada, eliminar todos los tokens
      $database->delete('poll_private_tokens')
        ->condition('poll_id', $poll_id)
        ->execute();
      $this->messenger()->addStatus($this->t('Poll is now public. All access tokens have been removed.'));
      return;
    }

    // Procesar nuevos emails
    $emails = $form_state->getValue('emails');
    if (!empty($emails)) {
      $emails = explode("\n", $emails);
      $emails = array_map('trim', $emails);
      $emails = array_filter($emails);
      foreach ($emails as $email) {
        if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
          $this->messenger()->addError($this->t('Invalid email: @email', ['@email' => $email]));
          continue;
        }
        $token = Crypt::randomBytesBase64(32);
        $database->insert('poll_private_tokens')
          ->fields([
            'poll_id' => $poll_id,
            'email' => $email,
            'token' => $token,
            'used' => 0,
          ])
          ->execute();

        $this->sendTokenEmail($email, $token, $poll_id);
      }
    }

    $this->messenger()->addStatus($this->t('Private access settings updated.'));
  }

  protected function sendTokenEmail($email, $token, $poll_id) {
    $poll = Poll::load($poll_id);
    $mailManager = \Drupal::service('plugin.manager.mail');
    $url = \Drupal\Core\Url::fromRoute('entity.poll.canonical', ['poll' => $poll_id], [
      'absolute' => TRUE,
      'query' => ['token' => $token],
    ])->toString();

    $params = [
      'subject' => $this->t('Your access to private poll: @title', ['@title' => $poll->label()]),
      'body' => [
        $this->t('You have been granted access to a private poll.'),
        $this->t('Poll title: @title', ['@title' => $poll->label()]),
        $this->t('Your access token: @token', ['@token' => $token]),
        $this->t('Access link: @link', ['@link' => $url]),
      ],
    ];

    $mailManager->mail('poll_private', 'token', $email, 'en', $params);
  }
}